using System;
using System.Linq;
using System.Threading.Tasks;
using EventosWeb.DataContext;
using EventosWeb.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace EventosWeb.Controllers
{
    [Route("api/v1/[controller]")] 
    [ApiController] 
    public class EventosController : ControllerBase
    {
        private readonly ApplicationDbContext _context;

    public EventosController(ApplicationDbContext context)
    {
        _context = context;
    }

    [HttpGet]
    public ActionResult GetEventos()
    {
        var eventos = _context.Eventos.ToList();
        return Ok(eventos);
    }

    [HttpGet("{id}")]
    public async Task<IActionResult> GetEvento(int id)
    {
        var evento = await _context.Eventos.FirstOrDefaultAsync(z => z.Id == id);

        if(evento == null)
            return NotFound();

        return Ok(evento);
    }

    [HttpPost]
    public async Task<IActionResult> CreateEvento(Evento data)
    {
        if(ModelState.IsValid)
        {
           await _context.Eventos.AddAsync(data);
           await _context.SaveChangesAsync();

            return CreatedAtAction("GetEvento", new {data.Id}, data);
        }

         return new JsonResult("Algo deu errado...") {StatusCode = 500};
    }

    [HttpPut("{id}")]
    public async Task<IActionResult> UpdateEvento(int id, Evento evento)
    {
        if(id != evento.Id)
            return BadRequest();
            
            if (evento is null)
            {
                throw new ArgumentNullException(nameof(evento));
            }

            var eventoExistente = await _context.Eventos.FirstOrDefaultAsync(z => z.Id == id);

        if(eventoExistente == null)
            return NotFound();

        eventoExistente.Titulo = evento.Titulo;
        eventoExistente.Descricao = evento.Descricao;
        eventoExistente.DataEvento = evento.DataEvento;
        eventoExistente.Status = evento.Status;

        await _context.SaveChangesAsync();

        return NoContent();
    }

    [HttpDelete("{id}")]
    public async Task<IActionResult> DeleteEvento(int id)
    {
        var evento = await _context.Eventos.FirstOrDefaultAsync(z => z.Id == id);

        if(evento == null)
            return NotFound();

        _context.Eventos.Remove(evento);
        await _context.SaveChangesAsync();

        return Ok(evento);
    }
  }
}